@@ -1,11 +1,8 @@
-__version__ = "0.0.5"
+__version__ = "0.0.7"
 from .Bodies import Body, Connector
 from .Joints import Joint
-from .Module import *
-from .Robot import *
-
+from .Module import AtomicModule, ModuleAssembly, ModuleBase, ModuleHeader, ModulesDB
+from .Robot import PinRobot, RobotBase
 from .task import Constraints, CostFunctions, Goals, Obstacle, Solution, Task, Tolerance
 from .utilities import errors, visualization
-
-from .utilities.tolerated_pose import ToleratedPose
 from .utilities.transformation import Transformation
